@@ -9,12 +9,35 @@ use mro 'c3'; # implements next::method
 
 use Params::Validate  qw/validate SCALAR SCALARREF CODEREF ARRAYREF HASHREF
                                   UNDEF  BOOLEAN/;
-use Scalar::Util      qw/blessed/;
-use Scalar::Does      qw/does/;
+use Scalar::Util      qw/blessed reftype/;
 use Carp;
+
+our $VERSION = '1.26';
+
+#----------------------------------------------------------------------
+# utility function : cheap version of Scalar::Does (too heavy to be included)
+#----------------------------------------------------------------------
+my %meth_for = (
+  ARRAY => '@{}',
+  HASH  => '%{}',
+ );
+
+sub does {
+  my ($data, $type) = @_;
+  my $reft = reftype $data;
+  return defined $reft && $reft eq $type
+      || blessed $data && overload::Method($data, $meth_for{$type});
+}
+
+#----------------------------------------------------------------------
+# remove all previously defined functions
+#----------------------------------------------------------------------
 use namespace::clean;
 
-our $VERSION = '1.17';
+
+#----------------------------------------------------------------------
+# global variables
+#----------------------------------------------------------------------
 
 # builtin methods for "Limit-Offset" dialects
 my %limit_offset_dialects = (
@@ -114,10 +137,14 @@ my %params_for_update = (
   -table        => {type => SCALAR},
   -set          => {type => HASHREF},
   -where        => {type => SCALAR|ARRAYREF|HASHREF, optional => 1},
+  -order_by     => {type => SCALAR|ARRAYREF|HASHREF, optional => 1},
+  -limit        => {type => SCALAR,                  optional => 1},
 );
 my %params_for_delete = (
   -from         => {type => SCALAR},
   -where        => {type => SCALAR|ARRAYREF|HASHREF, optional => 1},
+  -order_by     => {type => SCALAR|ARRAYREF|HASHREF, optional => 1},
+  -limit        => {type => SCALAR,                  optional => 1},
 );
 
 
@@ -126,7 +153,8 @@ my %params_for_delete = (
 #----------------------------------------------------------------------
 
 sub new {
-  my ($class, %params) = @_;
+  my $class = shift;
+  my %params = does($_[0], 'HASH') ? %{$_[0]} : @_;
 
   # extract params for this subclass
   my %more_params;
@@ -201,11 +229,15 @@ sub select {
   my @post_select;
   push @post_select, shift @cols while @cols && $cols[0] =~ s/^-//;
   foreach my $col (@cols) {
-    # extract alias, if any (recognized as "column|alias")
-    ($col, my $alias) = split /\|/, $col, 2;
-    if ($alias) {
-      $aliased_columns{$alias} = $col;
-      $col = $self->column_alias($col, $alias);
+    # extract alias, if any
+    if ($col =~ /^\s*         # ignore insignificant leading spaces
+                 (.*[^|\s])   # any non-empty string, not ending with ' ' or '|'
+                 \|           # followed by a literal '|'
+                 (\w+)        # followed by a word (the alias))
+                 \s*          # ignore insignificant trailing spaces
+                 $/x) {
+      $aliased_columns{$2} = $1;
+      $col = $self->column_alias($1, $2);
     }
   }
   $args{-columns} = \@cols;
@@ -278,7 +310,7 @@ sub select {
   }
 
   # add LIMIT/OFFSET if needed
-  if ($args{-limit}) {
+  if (defined $args{-limit}) {
     my ($limit_sql, @limit_bind) 
       = $self->limit_offset(@args{qw/-limit -offset/});
     $sql = $limit_sql =~ /%s/ ? sprintf $limit_sql, $sql
@@ -349,31 +381,62 @@ sub update {
   my $self = shift;
 
   my @old_API_args;
+  my %args;
   if (&_called_with_named_args) {
-    my %args = validate(@_, \%params_for_update);
+    %args = validate(@_, \%params_for_update);
     @old_API_args = @args{qw/-table -set -where/};
   }
   else {
     @old_API_args = @_;
   }
 
-  # return $self->next::method(@old_API_args);
-  return $self->_overridden_update(@old_API_args);
+  # call clone of parent method
+  my ($sql, @bind) = $self->_overridden_update(@old_API_args);
+
+  # maybe need to handle additional args
+  $self->_handle_additional_args_for_update_delete(\%args, \$sql, \@bind);
+
+  return ($sql, @bind);
 }
 
+
+sub _handle_additional_args_for_update_delete {
+  my ($self, $args, $sql_ref, $bind_ref) = @_;
+
+  if (defined $args->{-order_by}) {
+    my ($sql_ob, @bind_ob) = $self->_order_by($args->{-order_by});
+    $$sql_ref .= $sql_ob;
+    push @$bind_ref, @bind_ob;
+  }
+  if (defined $args->{-limit}) {
+    # can't call $self->limit_offset(..) because there shouldn't be any offset
+    $$sql_ref .= $self->_sqlcase(' limit ?');
+    push @$bind_ref, $args->{-limit};
+  }
+}
+
+
+
 sub delete {
   my $self = shift;
 
   my @old_API_args;
+  my %args;
   if (&_called_with_named_args) {
-    my %args = validate(@_, \%params_for_delete);
+    %args = validate(@_, \%params_for_delete);
     @old_API_args = @args{qw/-from -where/};
   }
   else {
     @old_API_args = @_;
   }
 
-  return $self->next::method(@old_API_args);
+  # call parent method
+  my ($sql, @bind) = $self->next::method(@old_API_args);
+
+  # maybe need to handle additional args
+  $self->_handle_additional_args_for_update_delete(\%args, \$sql, \@bind);
+
+  return ($sql, @bind);
 }
 
 #----------------------------------------------------------------------
@@ -526,23 +589,39 @@ sub _parse_join_spec {
   $bracket   ||= '{';
   $cond_list ||= '';
 
-  # find join syntax corresponding to the join operator
-  $self->{join_syntax}{$op}
-    or croak "unknown join operator : $op";
+  # extract constants (strings between quotes), replaced by placeholders
+  my $regex = qr/'       # initial quote
+                 (       # begin capturing group
+                  [^']*    # any non-quote chars
+                  (?:        # begin non-capturing group
+                     ''        # pair of quotes
+                     [^']*     # any non-quote chars
+                  )*         # this non-capturing group 0 or more times
+                 )       # end of capturing group
+                 '       # ending quote
+                /x;
+  my $placeholder = '_?_'; # unlikely to be counfounded with any value 
+  my @constants;
+  while ($cond_list =~ s/$regex/$placeholder/) {
+    push @constants, $1;
+  };
+  s/''/'/g for @constants;  # replace pairs of quotes by single quotes
 
   # accumulate conditions as pairs ($left => \"$op $right")
   my @conditions;
   foreach my $cond (split /,/, $cond_list) {
-    # parse the comparison operator (left and right operands + cmp op)
+    # parse the condition (left and right operands + comparison operator)
     my ($left, $cmp, $right) = split /([<>=!^]{1,2})/, $cond
       or croak "can't parse join condition: $cond";
 
-    # if operands are not qualified by table/alias name, add placeholders
+    # if operands are not qualified by table/alias name, add sprintf hooks
     $left  = "%1\$s.$left"  unless $left  =~ /\./;
-    $right = "%2\$s.$right" unless $right =~ /\./;
+    $right = "%2\$s.$right" unless $right =~ /\./ or $right eq $placeholder;
 
-    # add this pair into the list
-    push @conditions, $left, {$cmp => {-ident => $right}};
+    # add this pair into the list; right operand is either a bind value
+    # or an identifier within the right table
+    $right = $right eq $placeholder ? shift @constants : {-ident => $right};
+    push @conditions, $left, {$cmp => $right};
   }
 
   # list becomes an arrayref or hashref (for SQLA->where())
@@ -830,7 +909,7 @@ SQL::Abstract::More - extension of SQL::Abstract with more constructs and more f
 
 =head1 DESCRIPTION
 
-Generates SQL from Perl datastructures.  This is a subclass of
+Generates SQL from Perl data structures.  This is a subclass of
 L<SQL::Abstract>, fully compatible with the parent class, but with
 some additions :
 
@@ -931,13 +1010,13 @@ class (see L<SQL::Abstract/new>), plus the following :
 
 =item table_alias
 
-A L<sprintf> format description for generating table aliasing clauses.
+A C<sprintf> format description for generating table aliasing clauses.
 The default is C<%s AS %s>.
 Can also be supplied as a method coderef (see L</"Overriding methods">).
 
 =item column_alias
 
-A L<sprintf> format description for generating column aliasing clauses.
+A C<sprintf> format description for generating column aliasing clauses.
 The default is C<%s AS %s>.
 Can also be supplied as a method coderef.
 
@@ -945,7 +1024,7 @@ Can also be supplied as a method coderef.
 
 Name of a "limit-offset dialect", which can be one of
 C<LimitOffset>, C<LimitXY>, C<LimitYX> or C<RowNum>; 
-see L<SQL::Abstract::Limit> for an explation of those dialects.
+see L<SQL::Abstract::Limit> for an explanation of those dialects.
 Here, unlike the L<SQL::Abstract::Limit> implementation,
 limit and offset values are treated as regular values,
 with placeholders '?' in the SQL; values are postponed to the
@@ -954,16 +1033,16 @@ C<@bind> list.
 The argument can also be a coderef (see below
 L</"Overriding methods">). That coderef takes C<$self, $limit, $offset>
 as arguments, and should return C<($sql, @bind)>. If C<$sql> contains
-C<%s>, it is treated as a L<sprintf> format string, where the original
+C<%s>, it is treated as a C<sprintf> format string, where the original
 SQL is injected into C<%s>.
 
 
 =item join_syntax
 
-A hashref where keys are abreviations for join
+A hashref where keys are abbreviations for join
 operators to be used in the L</join> method, and 
 values are associated SQL clauses with placeholders
-in L<sprintf> format. The default is described
+in C<sprintf> format. The default is described
 below under the L</join> method.
 
 =item join_assoc_right
@@ -1274,7 +1353,7 @@ a scalar or an arrayref, it is passed directly to the parent method
 a hashref, it is interpreted as a SQL clause "RETURNING .. INTO ..",
 as required in particular by Oracle. Hash keys are field names, and
 hash values are references to variables that will receive the
-results. Then it is the client code's responsability
+results. Then it is the client code's responsibility
 to use L<DBD::Oracle/bind_param_inout> for binding the variables
 and retrieving the results, but the L</bind_params> method in the
 present module is there for help. Example:
@@ -1298,17 +1377,22 @@ present module is there for help. Example:
   # positional parameters, directly passed to the parent class
   ($sql, @bind) = $sqla->update($table, \%fieldvals, \%where);
 
-  # named parameters, handled in this class 
+  # named parameters, handled in this class
   ($sql, @bind) = $sqla->update(
-    -table => $table,
-    -set   => {col => $val, ...},
-    -where => \%conditions,
+    -table    => $table,
+    -set      => {col => $val, ...},
+    -where    => \%conditions,
+    -order_by => \@order,
+    -limit    => $limit,
   );
 
 This works in the same spirit as the L</insert> method above.
-Named parameters to the C<update()> method are just syntactic sugar
-for better readability of the client's code; they are passed verbatim
-to the parent method.
+Positional parameters are supported for backwards compatibility
+with the old API; but named parameters should be preferred because
+they improve the readability of the client's code.
+
+Few DBMS would support parameters C<-order_by> and C<-limit>, but
+MySQL does -- see L<http://dev.mysql.com/doc/refman/5.6/en/update.html>.
 
 
 =head2 delete
@@ -1318,14 +1402,19 @@ to the parent method.
 
   # named parameters, handled in this class 
   ($sql, @bind) = $sqla->delete (
-    -from  => $table
-    -where => \%conditions,
+    -from     => $table
+    -where    => \%conditions,
+    -order_by => \@order,
+    -limit    => $limit,
+
   );
 
-Named parameters to the C<delete()> method are just syntactic sugar
-for better readability of the client's code; they are passed verbatim
-to the parent method.
+Positional parameters are supported for backwards compatibility
+with the old API; but named parameters should be preferred because
+they improve the readability of the client's code.
 
+Few DBMS would support parameters C<-order_by> and C<-limit>, but
+MySQL does -- see L<http://dev.mysql.com/doc/refman/5.6/en/update.html>.
 
 =head2 table_alias
 
@@ -1360,17 +1449,20 @@ a collection of joined tables with their join conditions.
 The following example gives an idea of the available syntax :
 
   ($sql, @bind) = $sqla->join(qw[
-     Table1|t1       ab=cd         Table2|t2
-                 <=>{ef>gh,ij<kl}  Table3
-                  =>{t1.mn=op}     Table4
+     Table1|t1       ab=cd                     Table2|t2
+                 <=>{ef>gh,ij<kl,mn='foobar'}  Table3
+                  =>{t1.op=qr}                 Table4
      ]);
 
 This will generate
 
   Table1 AS t1 INNER JOIN Table2 AS t2 ON t1.ab=t2.cd
-               INNER JOIN Table3       ON t2.ef>Table3.gh 
+               INNER JOIN Table3       ON t2.ef>Table3.gh
                                       AND t2.ij<Table3.kl
-                LEFT JOIN Table4       ON t1.mn=Table4.op
+                                      AND t2.mn=?
+                LEFT JOIN Table4       ON t1.op=Table4.qr
+
+with one bind value C<foobar>.
 
 More precisely, the arguments to C<join()> should be a list
 containing an odd number of elements, where the odd positions
@@ -1402,7 +1494,7 @@ instead of the simple string representation.
 A join specification is a string containing
 an optional I<join operator>, possibly followed
 by a pair of curly braces or square brackets
-containing the I<join conditions>. 
+containing the I<join conditions>.
 
 Default builtin join operators are
 C<< <=> >>, C<< => >>, C<< <= >>, C<< == >>,
@@ -1417,7 +1509,7 @@ SQL JOIN clauses :
 This operator table can be overridden through
 the C<join_syntax> parameter of the L</new> method.
 
-The join conditions is a comma-separated list
+The join conditions are a comma-separated list
 of binary column comparisons, like for example
 
   {ab=cd,Table1.ef<Table2.gh}
@@ -1427,14 +1519,25 @@ or may be implicit, in which case they will be filled
 automatically from the names of operands on the
 left-hand side and right-hand side of the join.
 
-In accordance with L<SQL::Abstract> common conventions, 
+Strings within quotes will be treated as bind values instead
+of column names; pairs of quotes within such values become
+single quotes. Ex.
+
+  {ab=cd,ef='foo''bar',gh<ij}
+
+becomes
+
+  ON Table1.ab=Table2.cd AND Table1.ef=? AND Table1.gh<Table2.ij
+  # bind value: "foo'bar"
+
+In accordance with L<SQL::Abstract> common conventions,
 if the list of comparisons is within curly braces, it will
 become an C<AND>; if it is within square brackets, it will
 become an C<OR>.
 
 Join specifications expressed as strings
 are converted into internal hashrefs with keys
-C<operator> and  C<condition>, like this :
+C<operator> and C<condition>, like this :
 
   {
     operator  => '<=>',
@@ -1443,15 +1546,30 @@ C<operator> and  C<condition>, like this :
   }
 
 The C<operator> is a key into the C<join_syntax> table; the associated
-value is a sprinf format string, with placeholders for the left and
+value is a C<sprintf> format string, with placeholders for the left and
 right operands, and the join condition.  The C<condition> is a
 structure suitable for being passed as argument to
 L<SQL::Abstract/where>.  Places where the names of left/right tables
-(or their aliases) are expected should be expressed as sprintf
-placeholders, i.e.  respectively C<%1$s> and C<%2$s>. In most cases
-the right-hand side of the condition should B<not> belong to
-the C<@bind> list, so this is why we need to use the C<-ident> operator
-from L<SQL::Abstract>.
+(or their aliases) are expected should be expressed as C<sprintf>
+placeholders, i.e.  respectively C<%1$s> and C<%2$s>. Usually the
+right-hand side of the condition refers to a column of the right
+table; in such case it should B<not> belong to the C<@bind> list, so
+this is why we need to use the C<-ident> operator from
+L<SQL::Abstract>. Only when the right-hand side is a string constant
+(string within quotes) does it become a bind value : for example
+
+  ->join(qw/Table1 {ab=cd,ef='foobar'}) Table2/)
+
+is parsed into
+
+  [ 'Table1',
+    { operator  => '<=>',
+      condition => { '%1$s.ab' => {'=' => {-ident => '%2$s.cd'}},
+                     '%1$s.ef' => {'=' => 'foobar'} },
+    },
+    'Table2',
+  ]
+
 
 Hashrefs for join specifications as shown above can be passed directly
 as arguments, instead of the simple string representation.
@@ -1531,7 +1649,7 @@ statements of shape C<"INSERT ... RETURNING ... INTO ...">
 (see L</insert> method above), or as a way to indicate specific
 datatypes to the database driver.
 
-==head2 is_bind_value_with_type
+=head2 is_bind_value_with_type
 
   my ($method, @args) = $sqla->is_bind_value_with_type($value);
 
